package no.fjas.wspv;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import no.fjas.wspv.util.ClassLoaderUtil;
import no.fjas.wspv.util.DomHelper;
import no.fjas.wspv.validators.MultiPartValidator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * Loads web service validation configuration and validator configurations.
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class ConfigParser {
	private static Log log = LogFactory.getLog(ConfigParser.class);

	private static ReadWriteLock lock = new ReentrantReadWriteLock();

	private static Map<String, String> availableValidators = new HashMap<String, String>();

	/**
	 * Parses the validation configuration xml and builds up a
	 * {@link java.util.Map Map} that holds all validation rules for all
	 * services.
	 * 
	 * @param inStream
	 *            the stream holding the configuration file
	 * @param resourceName
	 *            the name of the configuration file (for debugging purposes
	 *            only)
	 * @return a Map with
	 *         {@link no.fjas.wspv.ServiceConfig ServiceConfig}s
	 *         mapped by service name.
	 */
	public static Map<String, ServiceConfig> parseValidatorConfigs(
			InputStream inStream, String resourceName) {
		Map<String, ServiceConfig> configs = new HashMap<String, ServiceConfig>();
		Document doc = null;

		InputSource in = new InputSource(inStream);
		in.setSystemId(resourceName);

		// Map<String, String> dtdMappings = new HashMap<String, String>();
		// dtdMappings.put("-//WS-Validator//EN",
		// "http://henning.fjas.no/ws-validator/validation.dtd");

		doc = DomHelper.parse(in, null);

		if (doc != null) {
			NodeList serviceNodes = doc.getElementsByTagName("service");

			for (int i = 0; i < serviceNodes.getLength(); i++) {
				Element serviceElement = (Element) serviceNodes.item(i);

				ServiceConfig serviceConfig = readServiceConfig(serviceElement);
				configs.put(serviceConfig.getName(), serviceConfig);
			}

			return configs;
		} else {
			return null;
		}
	}

	/**
	 * XML helper method reading in one single &lt;service&gt; element.
	 * 
	 * @param serviceElement
	 *            the xml element containing a service validation definition
	 * @return a ServiceConfig with validators for each operation and part.
	 */
	private static ServiceConfig readServiceConfig(Element serviceElement) {
		ServiceConfig config = new ServiceConfig();
		
		String serviceName = serviceElement.getAttribute("name");
		config.setName(serviceName);
		
		String serviceValidateEverything = serviceElement.getAttribute("validateEverything");
		config.setValidateEverything(Boolean.parseBoolean(serviceValidateEverything));

		NodeList operations = serviceElement.getElementsByTagName("operation");
		readOperationElements(config, serviceName + "/", operations);

		return config;
	}

	/**
	 * Reads in all operation elements from a service definition.
	 * 
	 * @param config
	 *            the configuration object to load into
	 * @param path
	 *            the name of the current path to identify a unique validator
	 * @param operations
	 *            the operation elements to load
	 */
	private static void readOperationElements(ServiceConfig config,
			String path, NodeList operations) {
		for (int i = 0; i < operations.getLength(); i++) {
			Element operationElement = (Element) operations.item(i);

			String operationName = operationElement.getAttribute("name");

//			NodeList parts = operationElement.getElementsByTagName("part");
			NodeList parts = operationElement.getChildNodes();

			StringBuilder newPath = new StringBuilder(path);
			newPath.append(operationName);
			newPath.append("/");

			readPartElements(config, newPath.toString(), parts);
		}
	}

	/**
	 * Read part elements. Part elements can consists of params or another part
	 * element (see validation.dtd)
	 * 
	 * @param config
	 *            config the configuration object to load into
	 * @param path
	 *            the name of the current path to identify a unique validator
	 * @param parts
	 *            the part elements to load
	 */
	private static void readPartElements(ServiceConfig config, String path,
			NodeList parts) {
		for (int i = 0; i < parts.getLength(); i++) {
			Element partElement = (Element) parts.item(i);

			StringBuilder fullPath = new StringBuilder(path);
			fullPath.append(partElement.getAttribute("name"));

			String validatorType = partElement.getAttribute("type");
			if ((validatorType == null || validatorType.equals(""))
					&& log.isDebugEnabled())
				log.debug("Element validator type is null for node named "
						+ partElement.getNodeName() + "|"
						+ partElement.getAttribute("name") + "| ->"
						+ partElement);
			Validator validator = lookupValidator(validatorType);

			if (validatorType.equals("multipart")) {
				if (validator instanceof MultiPartValidator) {
					MultiPartValidator multipartValidator = (MultiPartValidator) validator;
					readMultiPartElement(partElement, multipartValidator);
				} else {
					// if this occurs, there has been a major programming error
					throw new ValidatorException(
							"Validator is not a multipart validator!");
				}
			} else {
				readPartParameters(partElement, validator);
			}

			config.addValidator(fullPath.toString(), validator);
		}
	}

	/**
	 * Reads a part element that contains other part elements into a
	 * {@link no.fjas.wspv.validators.MultiPartValidator MultiPartValidator}.
	 * 
	 * @param multiPartElement
	 *            the part element to read
	 * @param parentValidator
	 *            the validator to load configuration into
	 */
	private static void readMultiPartElement(Element multiPartElement,
			MultiPartValidator parentValidator) {
		if (log.isDebugEnabled())
			log.debug("fetching part elements from element |" + multiPartElement.getAttribute("name") + "|");
		
//		NodeList children = multiPartElement.getElementsByTagName("part");
		NodeList children = multiPartElement.getChildNodes();
		
		for (int i = 0; i < children.getLength(); i++) {
			log.debug("part element list has size: " + children.getLength());
			log.debug("i is " + i);
			Element child = (Element) children.item(i);
			String childName = child.getAttribute("name");
			String validatorType = child.getAttribute("type");
			if (log.isDebugEnabled())
				log.debug("childname is " + childName + ", type is " + validatorType);
			
			if ((validatorType == null || validatorType.equals(""))
					&& log.isDebugEnabled())
				log.debug("Element validator type is null: " + child);

			Validator validator = lookupValidator(validatorType);

			if (validatorType.equals("multipart")) {
				if (validator instanceof MultiPartValidator) {
					MultiPartValidator multipartValidator = (MultiPartValidator) validator;
					readMultiPartElement(child, multipartValidator);
				} else {
					// if this occurs, there has been a major programming error
					throw new ValidatorException(
							"Validator is not a multipart validator!");
				}
			} else {
				readPartParameters(child, validator);
			}
			parentValidator.addValidator(childName, validator);
		}
	}

	/**
	 * Read a part element and all of its supplied parameters.
	 * 
	 * @param partElement
	 *            the part element to read
	 * @param validator
	 *            the validator to load configuration parameters into
	 */
	private static void readPartParameters(Element partElement,
			Validator validator) {
		NodeList paramList = partElement.getChildNodes();
		for (int j = 0; j < paramList.getLength(); j++) {
			Element param = (Element) paramList.item(j);
			validator.addParameter(param.getAttribute("name"), param
					.getTextContent());
		}
	}

	/**
	 * Looks up a validator by type name.
	 * 
	 * @param validatorType
	 *            the type of validator
	 * @return an instance of a validator
	 */
	public static Validator lookupValidator(String validatorType) {
		loadListOfValidatorsLock();

		String validatorClassName = null;
		Lock readLock = lock.readLock();
		readLock.lock();
		try {
			validatorClassName = availableValidators.get(validatorType);
		} finally {
			readLock.unlock();
		}
		Validator validator = null;

		if (validatorClassName != null) {
			try {
				validator = (Validator) ClassLoaderUtil.loadClass(
						validatorClassName, ConfigParser.class).newInstance();
			} catch (InstantiationException e) {
				throw new ValidatorException(
						"Error while trying to load validator class of type: "
								+ validatorType, e);
			} catch (IllegalAccessException e) {
				throw new ValidatorException(
						"Error while trying to load validator class of type: "
								+ validatorType, e);
			} catch (ClassNotFoundException e) {
				throw new ValidatorException(
						"Error while trying to load validator class of type: "
								+ validatorType, e);
			}
		} else {
			if (log.isWarnEnabled())
				log.warn("No validator loaded for the type '" + validatorType
						+ "'");
		}

		return validator;
	}

	/**
	 * Loads the list of validators. This method is thread safe.
	 */
	private static void loadListOfValidatorsLock() {
		Lock readLock = lock.readLock();
		readLock.lock();
		try {
			if (availableValidators.isEmpty()) {
				// no validator types loaded, need to aquire writelock and load
				// list
				readLock.unlock();
				Lock writeLock = lock.writeLock();
				writeLock.lock();
				try {
					// need to recheck for null beacuse of possible interferring
					// threads
					if (availableValidators.isEmpty()) {
						// load the default list first
						loadListOfValidatorsNoLock(Constants.VALIDATORS_DEFAULT);
						// then load custom validators if it exists
						loadListOfValidatorsNoLock(Constants.VALIDATORS_CUSTOM);
					}
				} finally {
					readLock.lock();
					writeLock.unlock();
				}
			}
		} finally {
			readLock.unlock();
		}
	}

	/**
	 * Does the actual loading of the validator list. This is not thread safe
	 * and should only be called by
	 * {@link loadListOfValidatorsLock loadListOfValidatorsLock}.
	 */
	private static void loadListOfValidatorsNoLock(String configurationFile) {
		InputSource configSource = new InputSource(ClassLoaderUtil
				.getResourceAsStream(configurationFile, ConfigParser.class));

		if (configSource != null) {
			try {
				Document doc = DomHelper.parse(configSource);

				NodeList validatorList = doc.getElementsByTagName("validator");
				for (int i = 0; i < validatorList.getLength(); i++) {
					Element element = (Element) validatorList.item(i);

					String name = element.getAttribute("name");
					String implClass = element.getAttribute("class");

					availableValidators.put(name, implClass);
				}
			} catch (ValidatorException e) {
				if (configurationFile.equals(Constants.VALIDATORS_CUSTOM)) {
					// A failure during loading of custom validators is not 
					// critical. Usually thrown because of a non-existing file
					if (log.isWarnEnabled())
						log.warn("Could not load validator configuration from "
								+ configurationFile);
				} else {
					throw e;
				}
			}

		}
	}
}