package no.fjas.wspv;

import no.fjas.wspv.validators.MultiPartValidator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Helper class for validating a SOAP envelope wrapped in a
 * {@link org.w3c.dom.Document}.
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class ValidatorSupport {

	protected static Log log = LogFactory.getLog(ValidatorSupport.class);

	/**
	 * Validates a SOAP envelope. Currently only supporting SOAP 1.1
	 * 
	 * @param config
	 *            the service configuration to use when validating
	 * @param doc
	 *            the entire soap envelope
	 * @throws ValidationException
	 *             upon validation errors
	 */
	public static void validateEnvelope(ServiceConfig config, Document doc)
			throws ValidationException {
		String serviceName = config.getName();
		Element envelope = doc.getDocumentElement();
		if (envelope != null) {

			// OBS!!!!
			// Using SOAP 1.1 namespace
			NodeList bodyList = envelope.getElementsByTagNameNS(
					Constants.SOAP_NAMESPACE_11, "Body");

			if (bodyList == null) {
				// try SOAP 1.2 namespace
				bodyList = envelope.getElementsByTagNameNS(
						Constants.SOAP_NAMESPACE_12, "Body");
			}
				
			
			
			if (bodyList != null) {
				Element body = (Element) bodyList.item(0);
				if (body != null) {
					if (log.isDebugEnabled())
						log.debug("body fetched: " + body.getNodeName());

					Element operation = (Element) body.getFirstChild();
					if (operation != null) {
						String operationName = operation.getNodeName();
						if (log.isDebugEnabled())
							log.debug("operation.name: " + operationName);

						NodeList partElements = operation.getChildNodes();
						for (int i = 0; i < partElements.getLength(); i++) {
							Element part = (Element) partElements.item(i);

							validateElement(config, serviceName, operationName,
									part);
						}
					} else {
						if (log.isWarnEnabled())
							log.warn("No operation element found");
					}
				} else {
					if (log.isWarnEnabled())
						log.warn("No body element found");
				}
			} else {
				if (log.isWarnEnabled())
					log.warn("No body elements found within envelope");
			}
		}
	}

	/**
	 * Validates a single element
	 * 
	 * @param config
	 *            the service configuration
	 * @param serviceName
	 *            the name of the service
	 * @param operationName
	 *            the name of the operation
	 * @param part
	 *            the part element to validate
	 * @throws ValidationException
	 *             upon validation errors
	 */
	private static void validateElement(ServiceConfig config,
			String serviceName, String operationName, Element part)
			throws ValidationException {

		log.debug("Element class name is: " + part.getClass());
		
		StringBuilder requestPath = new StringBuilder(serviceName);
		requestPath.append("/");
		requestPath.append(operationName);
		requestPath.append("/");
		requestPath.append(part.getNodeName());

		Validator validator = config.getValidators()
				.get(requestPath.toString());

		if (validator != null) {
			if (log.isDebugEnabled()) {
				if (validator.getClass().equals(MultiPartValidator.class))
					log.debug("Validating multipart element with validator: "
							+ validator.getClass());
				else
					log.debug("Validate value: |" + part.getTextContent()
							+ "| with validator: " + validator.getClass());
			}
			
			
			// finally, calling the validate method
			validator.validate(part);

			if (log.isDebugEnabled())
				log.debug("Validation successful");
		} else {
			if (log.isWarnEnabled())
				log.warn("No validator defined for " + requestPath.toString()
						+ "(" + part.toString() + ")"
						+ ", skipping validation.");

			if (config.isValidateEverything()) {
				if (log.isWarnEnabled())
					log.warn("Validation aborted due to missing validator for" +
							" element "	+ requestPath.toString());
				throw new ValidationException("Invalid value");
			}
		}
	}

}