package no.fjas.wspv.validators;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.Validator;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Base class for range based validators. Provides methods for doing min/max
 * inclusive/exclusive range checks.
 * 
 * <p>
 * All Validators extending this class has the following parameters:
 * <ul>
 * <li>maxInclusive - the maximum inclusive value</li>
 * <li>minInclusive - the minimum inclusive value</li>
 * <li>maxExclusive - the maximum exclusive value</li>
 * <li>minExclusive - the minimum exclusive value</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public abstract class AbstractRangeValidator implements Validator {

	protected static Log log = LogFactory.getLog(AbstractRangeValidator.class);

	protected final static String PARAM_MAX_INCLUSIVE = "maxInclusive";

	protected final static String PARAM_MIN_INCLUSIVE = "minInclusive";

	protected final static String PARAM_MIN_EXCLUSIVE = "minExclusive";

	protected final static String PARAM_MAX_EXCLUSIVE = "maxExclusive";

	/**
	 * Returns the max inclusive comparable value
	 * @return 
	 */
	protected abstract Comparable getMaxInclusiveComparatorValue();

	/**
	 * Returns the min inclusive comparable value
	 * @return
	 */
	protected abstract Comparable getMinInclusiveComparatorValue();

	/**
	 * Returns the max exclusive comparable value
	 * @return
	 */
	protected abstract Comparable getMaxExclusiveComparatorValue();

	/**
	 * Returns the min exclusive comparable value
	 * @return
	 */
	protected abstract Comparable getMinExclusiveComparatorValue();

	/**
	 * Validates a value against specific ranges that has been set through
	 * configuration parameters during initialization. The value to compare must
	 * be a {@link java.lang.Comparable} in the same way as the max/min
	 * inclusive/exclusive parameters.
	 * 
	 * @param value
	 * @throws ValidationException
	 */
	protected void validate(Comparable value) throws ValidationException {

		if (value == null) {
			log.info("Invalid value, NULL");
			throw new ValidationException("Invalid value");
		}

		// only perform checks if the comparator value is set

		if ((getMinInclusiveComparatorValue() != null)
				&& (value.compareTo(getMinInclusiveComparatorValue()) < 0)) {
			log.info("Invalid value, Out of range - minInlusive");
			throw new ValidationException("Invalid value");
		}

		if ((getMaxInclusiveComparatorValue() != null)
				&& (value.compareTo(getMaxInclusiveComparatorValue()) > 0)) {
			log.info("Invalid value, Out of range - maxInlusive");
			throw new ValidationException("Invalid value");
		}

		if ((getMinExclusiveComparatorValue() != null)
				&& (value.compareTo(getMinExclusiveComparatorValue()) <= 0)) {
			log.info("Invalid value, Out of range - minExlusive");
			throw new ValidationException("Invalid value");
		}

		if ((getMaxExclusiveComparatorValue() != null)
				&& (value.compareTo(getMaxExclusiveComparatorValue()) >= 0)) {
			log.info("Invalid value, Out of range - maxExlusive");
			throw new ValidationException("Invalid value");
		}

		if (log.isDebugEnabled())
			log.debug("Validation of value '" + value + "' was successful!");
	}

}