package no.fjas.wspv.validators;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.w3c.dom.Element;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.ValidatorException;

/**
 * Field Validator that checks if the date supplied is within a specific range.
 * 
 * <p>type name = <strong>date</strong></p>
 * 
 * <p>
 * Parameters accepted:
 * <ul>
 * <li>dateFormat - the format of the dates (default is "yyyy.MM.dd")</li>
 * <li>maxInclusive - the maximum inclusive value</li>
 * <li>minInclusive - the minimum inclusive value</li>
 * <li>maxExclusive - the maximum exclusive value</li>
 * <li>minExclusive - the minimum exclusive value</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 * 
 * @see java.text.SimpleDateFormat 
 *
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class DateRangeValidator extends AbstractRangeValidator {

	private final static String DEFAULT_DATE_FORMAT = "yyyy.MM.dd";
	
	private final static String PARAM_DATE_FORMAT = "dateFormat";
	
	private String dateFormat;
	
	private SimpleDateFormat dateFormatter;
	
	private Date maxInclusive;

	private Date minInclusive;

	private Date maxExclusive;

	private Date minExclusive;

	@Override
	protected Comparable getMaxInclusiveComparatorValue() {
		return maxInclusive;
	}

	@Override
	protected Comparable getMinInclusiveComparatorValue() {
		return minInclusive;
	}

	@Override
	protected Comparable getMaxExclusiveComparatorValue() {
		return maxExclusive;
	}

	@Override
	protected Comparable getMinExclusiveComparatorValue() {
		return minExclusive;
	}

	/**
	 * Parses a date from a string
	 * @param date the date to parse
	 * @return the date
	 */
	private Date parseDate(String date) throws ValidationException {
		if (dateFormatter == null) {
			if (dateFormat == null)  
				dateFormatter = new SimpleDateFormat(DEFAULT_DATE_FORMAT);
			else
				dateFormatter = new SimpleDateFormat(dateFormat);
		}
		
		Date parsedDate = null;
		
		try {
			parsedDate = dateFormatter.parse(date);
		} catch (ParseException e) {
			throw new ValidationException(e);
		}
		
		return parsedDate;
	}
	
	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#addParameter(java.lang.String, java.lang.String)
	 */
	public void addParameter(String key, String value)
			throws ValidatorException {
		
		if (key == null || key.equals("") || value == null || value.equals(""))
			throw new ValidatorException("Invalid parameter");
		
		if (key.equals(PARAM_DATE_FORMAT)) {
			dateFormat = value;
		} else {
			Date dateValue;
			
			try {
				dateValue = parseDate(value);
			} catch (ValidationException e) {
				throw new ValidatorException("Invalid parameter",e);
			}
			
			if (key.equals(PARAM_MAX_EXCLUSIVE))
				maxExclusive = dateValue;
			else if (key.equals(PARAM_MAX_INCLUSIVE))
				maxInclusive = dateValue;
			else if (key.equals(PARAM_MIN_EXCLUSIVE))
				minExclusive = dateValue;
			else if (key.equals(PARAM_MIN_INCLUSIVE))
				minInclusive = dateValue;
			else 
				if (log.isWarnEnabled())
					log.warn("Ignoring unknown parameter " + key + "=" + value);
		}
	}

	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
		String value = element.getTextContent();
		if (value == null)
			throw new ValidationException("Invalid value");
		else
			super.validate(parseDate(value));
	}
}
