package no.fjas.wspv.validators;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.ValidatorException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

/**
 * Validator that checks if the double specified is within a certain range.
 * 
 * <p>type name = <strong>double</strong></p>
 * 
 * <p>
 * Parameters accepted:
 * <ul>
 * <li>maxInclusive - the maximum inclusive value</li>
 * <li>minInclusive - the minimum inclusive value</li>
 * <li>maxExclusive - the maximum exclusive value</li>
 * <li>minExclusive - the minimum exclusive value</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class DoubleRangeValidator extends AbstractRangeValidator {

	protected static Log log = LogFactory
			.getLog(DoubleRangeValidator.class.getName());

	private Double maxInclusive = null;

	private Double minInclusive = null;

	private Double minExclusive = null;

	private Double maxExclusive = null;

	/**
	 * Parse a String to a Double
	 * 
	 * @param value the value to parse
	 * @return a Double representation of the value or null if not parseable.
	 * @throws ValidationException 
	 */
	private Double parseDouble(String value) throws ValidationException {
		Double doubleValue = null;
		if (value != null) {
			try {
				doubleValue = Double.valueOf(value);
			} catch (NumberFormatException e) {
				if (log.isWarnEnabled()) {
					log.warn("Unable to parse double from value " + value);
				}
				throw new ValidationException("Invalid value",e);
			}
		}
		return doubleValue;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see no.fjas.henning.xfire.validator.Validator#addParameter(java.lang.String,
	 *      java.lang.String)
	 */
	public void addParameter(String key, String value)
			throws ValidatorException {

		if (key == null || key.equals("") || value == null || value.equals(""))
			throw new ValidatorException("Invalid parameter");
		
		Double doubleValue;
		
		try {
			doubleValue = parseDouble(value);
		} catch (ValidationException e) {
			throw new ValidatorException(e);
		}

		if (doubleValue == null)
			throw new ValidatorException("Invalid parameter, not a parseable double");
		
		if (key.equals(PARAM_MAX_EXCLUSIVE)) {
			maxExclusive = doubleValue;
		} else if (key.equals(PARAM_MIN_EXCLUSIVE)) {
			minExclusive = doubleValue;
		} else if (key.equals(PARAM_MAX_INCLUSIVE)) {
			maxInclusive = doubleValue;
		} else if (key.equals(PARAM_MIN_INCLUSIVE)) {
			minInclusive = doubleValue;
		} else {
			if (log.isWarnEnabled())
				log.warn("Ignoring unknown parameter " + key + "=" + value);
		}
	}

	@Override
	protected Comparable getMaxInclusiveComparatorValue() {
		return maxInclusive;
	}

	@Override
	protected Comparable getMinInclusiveComparatorValue() {
		return minInclusive;
	}

	@Override
	protected Comparable getMaxExclusiveComparatorValue() {
		return maxExclusive;
	}

	@Override
	protected Comparable getMinExclusiveComparatorValue() {
		return minExclusive;
	}

	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
		String value = element.getTextContent();
		if (value == null)
			throw new ValidationException("Invalid value");
		else
			super.validate(parseDouble(value));
	}
}
