package no.fjas.wspv.validators;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.ValidatorException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

/**
 * Validator that checks if the integer specified is within a certain range.
 * 
 * <p>type name = <strong>int</strong></p>
 * 
 * <p>
 * Parameters accepted:
 * <ul>
 * <li>maxInclusive - the maximum inclusive value</li>
 * <li>minInclusive - the minimum inclusive value</li>
 * <li>maxExclusive - the maximum exclusive value</li>
 * <li>minExclusive - the minimum exclusive value</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class IntRangeValidator extends AbstractRangeValidator {

	protected static Log log = LogFactory.getLog(IntRangeValidator.class);

	private Integer maxInclusive = null;

	private Integer minInclusive = null;

	private Integer maxExclusive = null;

	private Integer minExclusive = null;
	
	/**
	 * Parses a String to an Integer
	 * 
	 * @param value
	 *            the value to parse
	 * @return an Integer representation of the value or null if not parseable.
	 */
	private Integer parseInt(String value) throws ValidationException {
		Integer intValue = null;
		try {
			intValue = Integer.valueOf(value);
		} catch (NumberFormatException e) {
			if (log.isDebugEnabled())
				log.debug("Unable to parse integer from value " + value);
			throw new ValidationException(e);
		}
		return intValue;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see no.fjas.henning.xfire.validator.Validator#addParameter(java.lang.String,
	 *      java.lang.String)
	 */
	public void addParameter(String key, String value)
			throws ValidatorException {

		if (key == null || key.equals("") || value == null || value.equals(""))
			throw new ValidatorException("Invalid parameter");
		
		Integer intValue;
		try {
			intValue = parseInt(value);
		} catch (ValidationException e) {
			throw new ValidatorException("Invalid parameter", e);
		}
		
		if (key.equals(PARAM_MAX_INCLUSIVE)) {
			maxInclusive = intValue;
		} else if (key.equals(PARAM_MIN_INCLUSIVE)) {
			minInclusive = intValue;
		} else if (key.equals(PARAM_MAX_EXCLUSIVE)) {
			maxExclusive = intValue;
		} else if (key.equals(PARAM_MIN_EXCLUSIVE)) {
			minExclusive = intValue;			
		} else {
			if (log.isWarnEnabled())
				log.warn("Ignoring unknown parameter " + key + "=" + value);
		}
	}

	@Override
	public Comparable getMaxInclusiveComparatorValue() {
		return maxInclusive;
	}

	@Override
	public Comparable getMinInclusiveComparatorValue() {
		return minInclusive;
	}

	@Override
	protected Comparable getMaxExclusiveComparatorValue() {
		return maxExclusive;
	}

	@Override
	protected Comparable getMinExclusiveComparatorValue() {
		return minExclusive;
	}

	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
		String value = element.getTextContent();
		if (value == null)
			throw new ValidationException("Invalid value");
		else
			super.validate(parseInt(value));
	}
}
