package no.fjas.wspv.validators;

import java.util.HashMap;
import java.util.Map;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.Validator;
import no.fjas.wspv.ValidatorException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The MultiPartValidator can validate an object that holds properties and their
 * respectable validators instead of just validating a single value.
 * 
 * <p>type name = <strong>multipart</strong></p>
 * 
 * <p>A configuration example:
 * <pre><code>
 * <validation>
 *	<service name="BookService">
 *		<operation name="addBook">
 *			<part name="in0" type="multipart">
 *				<part name="id" type="string"/>
 *				<part name="author" type="string"/>
 *				<part name="isbn" type="string"/>
 *				<part name="price" type="string"/>
 *				<part name="title" type="string"/>
 *			</part>
 *		</operation>
 *	</service>
 * </validation>
 * </code></pre>
 * </p>
 * 
 * <p>
 * Parameters accepted:
 * <ul>
 * <li>-none-</li>
 * </ul>
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class MultiPartValidator implements Validator {

	protected static Log log = LogFactory.getLog(MultiPartValidator.class);

	private Map<String, Validator> validators = new HashMap<String, Validator>();

	/*
	 * (non-Javadoc)
	 * 
	 * @see no.fjas.wspv.Validator#addParameter(java.lang.String,
	 *      java.lang.String)
	 */
	public void addParameter(String key, String value)
			throws ValidatorException {
		throw new ValidatorException(
				"addParameter method not supported on a multipart validator.");

	}

	/**
	 * Add single part validators to the multi part validator
	 * 
	 * @param elementName
	 *            name of the element to be validated
	 * @param validator
	 *            the validator to validate with
	 */
	public void addValidator(String elementName, Validator validator) {
		if (log.isDebugEnabled())
			log.debug("Adding multipart validator " + validator.getClass()
					+ " for element '" + elementName + "'");
		validators.put(elementName, validator);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
		if (element == null)
			throw new ValidationException("Invalid value");
		
		NodeList children = element.getChildNodes();
		if (children == null || children.getLength() == 0)
			throw new ValidationException("Invalid value");
		
		for (int i = 0; i < children.getLength(); i++) {
			Element child = (Element) children.item(i);
			Validator validator = validators.get(child.getNodeName());
			
			if (log.isDebugEnabled())
				log.debug("MultipartValidator - Validating " + child.getNodeName() + " with validator " + validator.getClass());

			if (validator != null) {
				validator.validate(child);
			} else if (log.isWarnEnabled()) {
				log.warn("No validator defined for multipart element '"
						+ child.getNodeName() + "'");
			}
		}
	}

}