package no.fjas.wspv.validators;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.Validator;
import no.fjas.wspv.ValidatorException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

/**
 * Validates a string using a regular expression.
 * 
 * <p>type name = <strong>regex</strong></p>
 * 
 * <p>
 * All Validators extending this class has the following parameters:
 * <ul>
 * <li>expression - the regular expression to match with</li>
 * <li>caseSensitive - if matching should be case sensitive (default is true)</li>
 * <li>trim - if the string should be trimmed first (default is true)</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 *
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 */
public class RegexValidator implements Validator {
	protected static Log log = LogFactory.getLog(RegexValidator.class);
	
	private final static String PARAM_EXPRESSION = "expression";
	private final static String PARAM_CASE_SENSITIVE = "caseSensitive";
	private final static String PARAM_TRIM = "trim";
	
    private String expression;
    private boolean caseSensitive = true;
    private boolean trim = true;

   /**
     * Sets the regular expression to be matched.
     */
    protected void setExpression(String expression) {
        this.expression = expression;
    }

    /**
     * Sets whether the expression should be matched against in
     * a case-sensitive way.  Default is <code>true</code>.
     */
    protected void setCaseSensitive(boolean caseSensitive) {
        this.caseSensitive = caseSensitive;
    }

   /**
     * Sets whether the expression should be trimed before matching.
     * Default is <code>true</code>.
     */
    protected void setTrim(boolean trim) {
        this.trim = trim;
    }

	/* (non-Javadoc)
	 * @see no.fjas.henning.xfire.validator.Validator#addParameter(java.lang.String, java.lang.String)
	 */
	public void addParameter(String key, String value) throws ValidatorException {
		
		if (key == null || key.equals("") || value == null || value.equals(""))
			throw new ValidatorException("Invalid parameter");
		
		if (key.equals(PARAM_EXPRESSION)) {
			setExpression(value);
		} else if (key.equals(PARAM_CASE_SENSITIVE)) {
			setCaseSensitive(Boolean.parseBoolean(value));
		} else if (key.equals(PARAM_TRIM)) {
			setTrim(Boolean.parseBoolean(value));
		} else {
			log.warn("Ignoring unrecognized parameter " + key);
		}		
	}

	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
        if (expression == null) {
        	log.info("expression is null, cannot validate");
        	throw new ValidationException("Invalid value");	
        }

		String value = element.getTextContent();
		
		if (value == null)
			throw new ValidationException("Invalid value");

        String str = ((String) value).trim();
        if (str.length() == 0) {
        	throw new ValidationException("Invalid value");
        }

        Pattern pattern;
        if (caseSensitive) {
            pattern = Pattern.compile(expression);
        } else {
            pattern = Pattern.compile(expression, Pattern.CASE_INSENSITIVE);
        }

        String compareValue = (String) value;
        if (trim) {
            compareValue = compareValue.trim();
        }
        
        Matcher matcher = pattern.matcher(compareValue);

        if (!matcher.matches()) {
        	if (log.isDebugEnabled())
        		log.debug("Validation failed for value: |" + value + "| width expression: |" + expression + "|");
        	throw new ValidationException("Invalid value");
        } else {
        	if (log.isDebugEnabled())
        		log.debug("Validation passed for value: |" + value + "|");
        }		
	}
}