package no.fjas.wspv.validators;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import no.fjas.wspv.ValidationException;
import no.fjas.wspv.Validator;
import no.fjas.wspv.ValidatorException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Element;

/**
 * StringLengthValidator checks that a String is of a certain length.
 * 
 * <p>type name = <strong>string</strong></p>
 * 
 * <p>
 * Parameters accepted:
 * <ul>
 * <li>minLength - the minimum (exclusive) length</li>
 * <li>maxLength - the maximum (exclusive) length</li>
 * <li>trim - if the string should be trimmed (default is true)</li>
 * <li>allowEmpty - if the string can be empty i.e. "" (default is true)</li>
 * <li>allowHtml - if the string can contain html (default is true, i.e. no checking of html presens)</li>
 * </ul>
 * 
 * <em>Original class from OpenSymphony XWork, modified for use in WSPV.</em>
 * </p>
 * 
 * @author <a href="mailto:henning _at_ fjas.no">Henning Jensen</a>
 *
 */
public class StringLengthValidator implements Validator {
	private static Log log = LogFactory.getLog(StringLengthValidator.class);

	private int minLength = -1;

	private int maxLength = -1;

	private boolean trim = true;

	private boolean allowEmpty = true;

	private boolean allowHtml = false;

	private static final String PARAM_MIN_LENGTH = "minLength";

	private static final String PARAM_MAX_LENGTH = "maxLength";

	private static final String PARAM_TRIM = "trim";

	private static final String PARAM_ALLOW_EMPTY = "allowEmpty";

	private static final String PARAM_ALLOW_HTML = "allowHtml";

	private static final Pattern htmlPattern;
	
	/*
	 * Paranoid HTML matcher, searching for html start tags including hex codes
	 * http://www.securityfocus.com/infocus/1768 
	 * Unescaped raw version: ((\%3C)|<)[^\n]+((\%3E)|>)  
	 */
	private static final String REGEX_HTML = "((\\%3C)|<)[^\\n]+((\\%3E)|>)";
	

	/*
	 * Initializing the regular expression, no need doing this for each run
	 */
	static {
		htmlPattern = Pattern.compile(REGEX_HTML, Pattern.CASE_INSENSITIVE);
	}
	
	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#addParameter(java.lang.String, java.lang.String)
	 */
	public void addParameter(String key, String value)
			throws ValidatorException {
		
		if (key == null || key.equals("") || value == null || value.equals(""))
			throw new ValidatorException("Invalid parameter");
		
		try {
			if (key.equals(PARAM_MIN_LENGTH))
				minLength = Integer.parseInt(value);
			else if (key.equals(PARAM_MAX_LENGTH))
				maxLength = Integer.parseInt(value);
			else if (key.equals(PARAM_TRIM))
				trim = Boolean.parseBoolean(value);
			else if (key.equals(PARAM_ALLOW_EMPTY))
				allowEmpty = Boolean.parseBoolean(value);
			else if (key.equals(PARAM_ALLOW_HTML))
				allowHtml = Boolean.parseBoolean(value);
			else if (log.isWarnEnabled())
				log.warn("Udefined parameter '" + key
						+ "' for StringLengthValidator, skipping");
		} catch (NumberFormatException e) {
			throw new ValidatorException("Invalid parameter",e);
		}
	}

	/* (non-Javadoc)
	 * @see no.fjas.wspv.Validator#validate(org.w3c.dom.Element)
	 */
	public void validate(Element element) throws ValidationException {
		String value = element.getTextContent();
//		String value = element.getNodeValue();
		if (value == null)
			throw new ValidationException("Invalid value");
	
		if (trim)
			value = value.trim();

		if (allowEmpty && value.equals(""))
			throw new ValidationException("Invalid value");

		if (!allowHtml) {
			Matcher m = htmlPattern.matcher(value);
			if (m.find()) {
				throw new ValidationException("Invalid value");
			}
		}

		if ((minLength > -1) && (value.length() < minLength)) {
			throw new ValidationException("Invalid value");
		} else if ((maxLength > -1) && (value.length() > maxLength)) {
			throw new ValidationException("Invalid value");
		}
	}
}
